"""Builtin environment for the analyzer.  Almost identical
to the builtins that the regular evaluator uses, but adjusts
some of the bindings where we have to do something different.
"""

import types
import math
import sys
from symbol import Symbol, true, false
import expressions
import environment
import pair
import pogo
from error import SchemeError
import operator
import analyzer
import parser

import builtins



def setupEnvironment():
    """Sets up a new environment with a bunch of fairly standard
    Scheme built-in primitives."""
    initial_env = builtins.setupEnvironment()
    for (name, value) in ( ('load', schemeLoad),
                           ('dir', schemeDir),
                           ('call-with-current-continuation', schemeCallcc),
                           ('call/cc', schemeCallcc)
                           ):
        environment.setVariableValue(
            Symbol(name),
            expressions.makePrimitiveProcedure(value),
            initial_env)
    return initial_env



def schemeDir(cont, env, args):
    """A quick function to inspect the first frame."""
    if len(args) != 0:
        raise TypeError, ("dir expected at most 0 arguments, got %s" % len(args))
    names = environment.firstFrame(env).keys()
    names.sort()
    return pogo.apply_cont(cont, pair.schemeList(* map(Symbol, names)))


def schemeLoad(cont, env, args):
    """Special primitive: implements LOAD."""
    symbolicFilename = str(args[0])
    try:
        f = open(symbolicFilename)
        try:
            text = "(begin \n%s\n 'ok)" % f.read()
        finally:
            f.close()
        return analyzer.texec(analyzer.analyze(parser.parse(text)),
                              env, cont)
    except IOError, e:
        raise SchemeError, "LOAD error -- %s" % str(e)
    except TypeError, e:
        raise SchemeError, "LOAD error -- argument must be a string"


def schemeCallcc(cont, env, args):
    analyzedLambdaBody = args[0]
    def c2(val):
        return pogo.apply_cont(cont, val)
    cont_procedure = expressions.makeContinuationProcedure(c2)
    return analyzer.apply(analyzedLambdaBody,
                          pair.schemeList(cont_procedure),
                          env,
                          cont)
